<?php
require_once("dll.php");
$variables->getSafeVars(array_keys($_REQUEST));
set_time_limit(0);

/************************ Start compatibility backup support for CMS old versions ******************************/
/************************** Allows to put this file as standalone in admin folder ********************************/
if (!method_exists($system, 'backup')) {
    function backup() { // create backup functionality for for older CMS versions
        if (basename(dirname($_SERVER["PHP_SELF"])) != "admin") die("Please run Backup from admin folder");
        $rootPath = realpath('../'); // must be called from admin folder
        $zip_filename = 'uploads/backup.zip';
        if (is_file($zip_filename)) unlink($zip_filename); // make sure we wont backup any previous backup file
        $zip = new ZipArchive();
        $zip->open($zip_filename, ZipArchive::CREATE | ZipArchive::OVERWRITE);
        $files = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($rootPath),
            RecursiveIteratorIterator::LEAVES_ONLY
        );
        foreach ($files as $name => $file) {
            if (!$file->isDir()) {
                $filePath = $file->getRealPath();
                $relativePath = substr($filePath, strlen($rootPath) + 1);
                $zip->addFile($filePath, $relativePath);
            }
        }
        $zip->close();
        $zip = file($zip_filename);
        return $zip;
    }
} else {
    function backup() {  // create a shortcut to system's backup method for newer CMS versions
        global $system;
        return $system->backup();
    }
}

if (!method_exists($system, 'backupDatabase')) {
    function backupDatabase() { // create backupDatabase functionality for older CMS versions
        unlink('uploads/backup.sql'); // delete any previous dump (path from admin folder)
        if (basename(dirname($_SERVER["PHP_SELF"])) != "admin") die("Please run Database Backup from admin folder");
        global $dbHost, $dbName, $dbUser, $dbPass;
        require_once("mysqlDump/Mysqldump.php"); // path from admin folder
        $dumpSettings = array(
            'compress' => 'None',
            'add-drop-table' => true,
            'default-character-set' => 'utf8mb4',
        );
        $dump = new Ifsnop\Mysqldump\Mysqldump("mysql:host=$dbHost;dbname=$dbName", "$dbUser", "$dbPass", $dumpSettings);
        $dump->start('uploads/backup.sql');
    }
} else {
    function backupDatabase() {  // create a shortcut to system's backupDatabase method for newer CMS versions
        global $system;
        return $system->backupDatabase();
    }
}

if (!method_exists($system, 'getUpdateInfo')) {
    if (!function_exists('getUpdateInfo')) {
        function getUpdateInfo() { // create getUpdateInfo functionality for older CMS versions
            global $eshop;
            global $currentVersion;
            $cms = is_object($eshop) ? "odyshop" : "odyssey";
            $ch = curl_init();
            $url = "http://odyssey.webpage.gr/new/updates/api/?cms=$cms&action=next_version&from_version=$currentVersion";
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 1);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_POST, 1);
            $data = curl_exec($ch);
            curl_close($ch);
            return json_decode($data);
        }
    }
} else {
    if (!function_exists('getUpdateInfo')) {
        function getUpdateInfo() {  // create a shortcut to system's getUpdateInfo method for newer CMS versions
            global $system;
            return $system->getUpdateInfo();
        }
    }
}

/************************ End compatibility backup support for CMS old versions ******************************/

if ($method == "ajax") {
    if ($action == "check_for_updates") {
        $update_data = getUpdateInfo();
        $update_file = $update_data->link;
        if ((float)$update_data->version > (float)$currentVersion) {
            if (!$update_file) die("Δεν ήταν δυνατή η ανάκτηση του απομακρυσμένου αρχείου ενημέρωσης");
            if ($update_data->messages) {
                die(implode(",", $update_data->messages));
            } else {
                die("ok_" . $update_data->version);
            }
        } else {
            die(t("Το σύστημά σας είναι ενημερωμένο!"));
        }
    }
    if ($action == "backup_database") {
        backupDatabase(); // backup database
        if (is_file('uploads/backup.sql')) {
            die("ok");
        } else {
            die(t("Σφάλμα κατά την δημιουργία του αντιγράφου ασφαλείας της βάσης δεδομένων"));
        }
    }
    if ($action == "backup_files") {
        $zip = backup(); // backup files
        if ($zip) {
            die("ok");
        } else {
            die(t("Σφάλμα κατά την δημιουργία του αντιγράφου ασφαλείας των αρχείων του site"));
        }
    }
    if ($action == "download_update") {
        $update_data = getUpdateInfo();
        $update_file = $update_data->link;
        if ((float)$update_data->version > (float)$currentVersion) {
            if (!$update_file) die("Δεν ήταν δυνατή η ανάκτηση του απομακρυσμένου αρχείου ενημέρωσης");
            if ($update_data->messages) die(implode(",", $update_data->messages));
            $dest_file = realpath('.') . '/uploads/' . basename($update_file);
            if (copy($update_file, $dest_file)) {
                $zip = new ZipArchive;
                $res = $zip->open($dest_file);
                if ($res === TRUE) {
                    if ($zip->extractTo('../')) {
                        $zip->close();
                        unlink($dest_file); // cleaning up...
                        die("ok");
                    } else {
                        die(t("Δεν ήταν δυνατή η αποσυμπίεση του zip αρχείου ενημέρωσης"));
                    }
                } else {
                    die(t("Δεν ήταν δυνατό το άνοιγμα του zip αρχείου ενημέρωσης"));
                }
            } else {
                die(t("Δεν ήταν δυνατή η αντιγραφή του απομακρυσμένου αρχείου ενημέρωσης"));
            }
        } else {
            die(t("Το σύστημά σας είναι ενημερωμένο!"));
        }
    }

    die("Error: No valid action defined");
}

?>
<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <link rel="shortcut icon" href="favicon.ico">
    <title>Admin Panel</title>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link href="css/admin.css" rel="stylesheet" type="text/css">
    <link href="css/pagination.css" rel="stylesheet" type="text/css">
    <link href="css/sort.css" rel="stylesheet" type="text/css">
    <link rel="stylesheet" type="text/css" href="js/fancybox2/fancybox.css">
    <link type="text/css" href="css/select2/select2.css" rel="stylesheet">
    <link type="text/css" href="css/ui/jquery-ui-odyssey.css" rel="stylesheet">
    <link rel="stylesheet" media="all" type="text/css" href="css/jquery-impromptu.css">

    <script type="text/javascript" src="js/jquery.js"></script>
    <script type="text/javascript" src="js/admin.js"></script>
    <script type="text/javascript" src="js/cookies.js"></script>
    <script type="text/javascript" src="js/jquery-ui-All.min.js"></script>
    <script type="text/javascript" src="js/fancybox2/fancybox.js"></script>
    <script type="text/javascript" src="js/select2.min.js"></script>
    <script type="text/javascript" src="js/jquery-impromptu.js"></script>
    <style type="text/css">
        .update_action {
            margin-bottom: 20px;
        }

        .update_action_icon {
            display: table-cell;
            vertical-align: middle;
        }

        .update_action_completed {
            display: inline-block;
            padding-left: 30px;
            background: url("images/active_checked.png") left center no-repeat;
            margin-top: 7px;
            font-weight: bold;
            color: #9a2a54;
        }

        .update-action-btn {
            margin-bottom: 20px;
        }

        .update_found_new_version {
            background: url("images/tip.png") left center no-repeat;
        }

        .update_action_error {
            background: url("images/warning-small.png") left center no-repeat;
            text-align: left;
            display: inline-block;
            padding: 20px 20px 20px 30px;
            color: #d50000;
            font-size: 18px;
            font-style: italic;
        }
    </style>
    <script>

        odyssey.update_has_error = false;

        jQuery(function ($) {
            $('#loading').css("display", "none");
            $('a[rel="open_sections"]').fancybox({
                width: 500,
                maxHeight: 500,
                fitToView: true,
                autoSize: false,
                type: 'iframe',
                arrows: false,
                helpers: {
                    overlay: {
                        css: {
                            'background': 'rgba(0, 0, 0, 0.8)'
                        }
                    },
                    title: null
                }
            });

            update_online();

        });

        function update_online() {

            $.when(
                $.ajax({
                    type: "POST",
                    url: "update_online.php",
                    data: "method=ajax&action=check_for_updates",
                    async: false,
                    success: function (response) {
                        response = response.trim();
                        console.log(response);
                        if (response.indexOf("ok_") != -1) {
                            var version = response.replace("ok_", "");
                            var html = '';
                            if (parseFloat(version)) html += "<div class='update_action'><span class='update_action_completed update_found_new_version'><?php echo t("Βρέθηκε νέα έκδοση"); ?> (v" + version + ")</span></div>";
                            html += "<div class='update_action'><?php echo t("Δημιουργία αντιγράφου ασφαλείας της βάσης δεδομένων, παρακαλώ περιμένετε...");?>" + "<div id='backup_database' class='update_action_icon'><img src='images/loading_small.gif'></div></div>";
                            $('#update_status').append(html);
                        } else {
                            html = "<div class='update_action_error'>" + response + "</div>";
                            $('#update_status').append(html);
                            odyssey.update_has_error = true;
                        }
                    },
                    error: function (response) {
                        html = "<div class='update_action_error'>Ajax error connecting to server</div>";
                        $('#update_status').append(html);
                        odyssey.update_has_error = true;
                    }
                })
            );
            if (odyssey.update_has_error) return false;
            $.when(
                $.ajax({
                    type: "POST",
                    url: "update_online.php",
                    data: "method=ajax&action=backup_database",
                    async: false,
                    success: function (response) {
                        response = response.trim();
                        console.log(response);
                        if (response == "ok") {
                            $('#backup_database').html('<span class="update_action_completed "><?php echo t("Ολοκληρώθηκε!");?></span>');
                            html = "<div class='update_action'><?php echo t("Δημιουργία αντιγράφου ασφαλείας των αρχείων του site, παρακαλώ περιμένετε...");?>" + "<div id='backup_files' class='update_action_icon'><img src='images/loading_small.gif'></div></div>";
                            $('#update_status').append(html);
                        } else {
                            html = "<div class='update_action_error'>" + response + "</div>";
                            $('#backup_database').html(html);
                            odyssey.update_has_error = true;
                        }
                    },
                    error: function (response) {
                        html = "<div class='update_action_error'>Ajax error connecting to server</div>";
                        $('#update_status').append(html);
                        odyssey.update_has_error = true;
                    }
                })
            );
            if (odyssey.update_has_error) return false;
            $.when(
                $.ajax({
                    type: "POST",
                    url: "update_online.php",
                    data: "method=ajax&action=backup_files",
                    async: false,
                    success: function (response) {
                        response = response.trim();
                        console.log(response);
                        if (response == "ok") {
                            $('#backup_files').html('<span class="update_action_completed"><?php echo t("Ολοκληρώθηκε!");?></span>');
                            html = "<a href='uploads/backup.zip' class='btn btn-info update-action-btn'><?php echo t("Λήψη αντιγράφου ασφαλείας");?></a>";
                            html += "<div class='update_action'><?php echo t("Λήψη του αρχείου ενημέρωσης, παρακαλώ περιμένετε...");?>" + "<div id='download_update' class='update_action_icon'><img src='images/loading_small.gif'></div></div>";
                            $('#update_status').append(html);
                        } else {
                            html = "<div class='update_action_error'>" + response + "</div>";
                            $('#backup_files').html(html);
                            odyssey.update_has_error = true;
                        }
                    },
                    error: function (response) {
                        html = "<div class='update_action_error'>Ajax error connecting to server</div>";
                        $('#backup_files').html(html);
                        odyssey.update_has_error = true;
                    }
                })
            );
            if (odyssey.update_has_error) return false;
            $.when(
                $.ajax({
                    type: "POST",
                    url: "update_online.php",
                    data: "method=ajax&action=download_update",
                    async: false,
                    success: function (response) {
                        response = response.trim();
                        console.log(response);
                        if (response == "ok") { // we have the update zip extracted!
                            $('#download_update').html('<span class="update_action_completed"><?php echo t("Ολοκληρώθηκε!");?></span>');
                            html = "<a href='index.php' class='btn btn-success update-action-btn'><?php echo t("Ενημέρωση συστήματος");?></a>";
                            $('#update_status').append(html);
                        } else {
                            html = "<div class='update_action_error'>" + response + "</div>";
                            $('#download_update').html(html);
                            odyssey.update_has_error = true;
                        }
                    },
                    error: function (response) {
                        html = "<div class='update_action_error'>Ajax error connecting to server</div>";
                        $('#download_update').html(html);
                        odyssey.update_has_error = true;
                    }
                })
            );
            return !odyssey.update_has_error;
        }
        //-->
    </script>
</head>
<body>
<div id="backend">
    <div id="sortInfo"></div>
    <?php include("includes/loading.php"); ?>
    <section class="container">

        <?php include("header.php"); ?>
        <div class="group">
            <aside class="grid-4">
                <?php include("menu.php"); ?>
                &nbsp;
            </aside>

            <article class="grid-8">
                <?php
                $messages->printSystemMessages();
                ?>
                <div class="section_title"><?php echo t("ΕΝΗΜΕΡΩΣΗ ΤΟΥ ΣΥΣΤΗΜΑΤΟΣ"); ?></div>
                <div class="main">
                    <div id="update_status"></div>
                </div>
            </article>
        </div>
        <?php include("footer.php"); ?>
    </section>
</div>
</body>
</html>