<?php

$rootPath = "";
$counter = 0;
$file = "factory.php"; // this file is unique in root
while ($counter <= 10) {
    if (is_file($rootPath . $file)) break;
    $counter++;
    $rootPath .= "../";

}

@include($rootPath . "custom_functions.php");

function get_libraries($rootPath = '', $libraries = array()) {
    foreach (glob($rootPath . "library/*.php") as $filename) {
        $library_name = str_replace(".php", "", basename($filename));
        if (in_array($library_name, $libraries)) require_once($filename);
    }
    foreach ($libraries as $library) {
        global ${$library};
        ${$library} = new $library();
    }
}

function checkInstallation($admin = false) {
    $path = ($admin) ? "../" : NULL;
    if (!is_file($path . "config/odyssey.ini.php")) {
        $installFile = $path . "admin/install.php";
        if (is_file($installFile)) {
            header('location:' . $installFile);
            exit;
        } else {
            die("Critical error! Odyssey installation file not found...");
        }
    }
}

function filterInvalidRequest() {
    if (preg_match('/\/index\.php\//', $_SERVER['REQUEST_URI']) || preg_match('/mediabank\/medium\/mediabank/', $_SERVER['REQUEST_URI'])) {
        die("Invalid URI");
    }
}

function setErrorReporting() {
    if (isset($_GET['debug_errors']) || $_SERVER['SERVER_NAME'] == "localhost") { //force or development error reporting!
        if (defined('E_DEPRECATED')) {
            error_reporting(E_ALL ^ E_NOTICE ^ E_WARNING ^ E_DEPRECATED);
        } else {
            error_reporting(E_ALL ^ E_NOTICE ^ E_WARNING);
        }
    } else {
        error_reporting(0); // normal behaviour for production
    }
}

function setSSL() {
    if ($_SERVER['SERVER_NAME'] == "localhost") return false; // no SSL for local server
    $secureURL = preg_replace("/^http:/i", "https:", "http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]");
    header('location: ' . $secureURL);
    exit;
}

function getHeaders() {
    session_set_cookie_params(NULL, NULL, NULL, false, true);
    session_start();
    mb_internal_encoding("utf-8");
    header('P3P:CP="IDC DSP COR ADM DEVi TAIi PSA PSD IVAi IVDi CONi HIS OUR IND CNT"');
    header('Content-type: text/html; charset=utf-8');
    $_SERVER['PHP_SELF'] = str_replace('"', '', htmlspecialchars(str_replace('script', '', $_SERVER['PHP_SELF']), ENT_QUOTES));
}

function importSession() {
    global $adminPass;
    if (isset($_GET['session_data']) && is_file("uploads/" . $_GET['session_data'])) { // try to import the session of sef url
        $file = fopen("uploads/" . $_GET['session_data'], "r+");
        $contents = fgets($file);
        fclose($file);
        $importedSession = unserialize($contents);
        $sessionValidated = false;
        foreach ($importedSession as $key => $value) {
            $_SESSION[$key] = $value;
            if ($key == "odykey" && $value == $adminPass) $sessionValidated = true;
        }
        if (!$sessionValidated) session_destroy(); // not valid session data tried to pass, so destroy the session
        unlink("uploads/" . $_GET['session_data']);
    }
}

function getProtocol() {
    if (isset($_GET['secure_protocol'])) {
        $protocol = ($_GET['secure_protocol'] == "1") ? "https" : "http";
    } else {
        $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https" : "http";
    }
    return $protocol;
}

function cleanUpUploads() {
    foreach (glob("uploads/*") as $filename) {
        $creationTimestamp = filemtime($filename);
        $timePast = time() - $creationTimestamp;
        if ($timePast > 1000 && is_file($filename)) unlink($filename);
    }
}

function __autoload($className) {
    global $imports;
    if (isset($imports[$className])) {
        include_once($imports[$className]);
    }
}

function import($import, $admin = false) {
    global $library;
    global $imports;
    $path = ($admin) ? "../" : NULL;
    $lastDot = strrpos($import, '.');
    $class = $lastDot ? substr($import, $lastDot + 1) : $import;
    $package = substr($import, 0, $lastDot);
    if (isset($imports[$class]) || isset($imports[$package . '.*'])) return true;
    $folder = $path . ($package ? str_replace('.', '/', $package) : '');
    $file = $path . $folder . "/" . $class . ".php";
    if (!file_exists($folder)) {
        $back = debug_backtrace();
        return trigger_error("There is no such package <strong>'$package'</strong> -- Checked folder <strong>'$folder'</strong><br>
            Imported from <strong>'{$back[0]['file']}'</strong> on line <strong>'{$back[0]['line']}'</strong><br>", E_USER_WARNING);
    } elseif ($class != '*' && !file_exists($file)) {
        $back = debug_backtrace();
        return trigger_error("There is no such Class <strong>'$import'</strong> -- Checked for file <strong>'$file'</strong><br>
            Imported from <strong>'{$back[0]['file']}'</strong> on line <strong>'{$back[0]['line']}'</strong><br>", E_USER_WARNING);
    }
    if ($class != '*') {
        $imports[$class] = $file;
    } else {
        $imports["$package.*"] = 1;
        $dir = opendir($folder);
        while (($file = readdir($dir)) !== false) {
            if (strrpos($file, '.php')) {
                $class = str_replace('.php', '', $file);
                $imports[$class] = $folder . "/" . $file;
                $library[] = $class;
            }
        }
    }
}

function getAdminLanguage() {
    global $db;
    global $language;
    if ($language->isMultilingualSite()) {
        $langID = $db->getRecord("select `admin_language` from `system_settings` limit 1")->admin_language;
        return ($langID) ? $langID : 1; // return greek language if no other language found in system settings
    } else { // single language, check if it's greek else set admin language to english
        $langID = $language->getDefaultLanguage(true);
        $ext = $language->getLanguageExtension($langID);
        if ($ext != "_gr") { // return english as admin language when the default language is not greek
            $langID = $language->getLanguageByExtension("_en");
        } else {
            $langID = $language->getLanguageByExtension("_gr"); // set greek as admin language
        }
        setAdminLanguage($langID);
        return $langID;
    }
}

function setAdminLanguage($langID) {
    if (!$langID) $langID = getAdminLanguage();
    global $db;
    $q = "update `system_settings` set admin_language='$langID'";
    $db->query($q);
}

function manageLogout() {
    global $db;
    global $auth;
    if (!empty($_REQUEST['logout'])) { // logout for non sef urls
        $userID = $_SESSION['userID'];
        $q = "delete from connected_users where userID = '$userID'";
        $db->query($q);
        $auth->logout();
    }
}

function manageMultilingual() {
    global $language;
    global $sef;
    global $parameters;
    global $variables;
    if ($language->isMultilingualSite() && $_SERVER["REQUEST_METHOD"] != "POST") {
        if (isset($_REQUEST['langID'])) {
            if (!$language->isLanguageActive((int)$_REQUEST['langID']) || !(int)$_REQUEST['langID']) {
                $language->setCurrentLanguage();
                header("location:" . $sef->url("index.php?langID=" . $language->getCurrentLanguage(), false));
                exit();
            } else {
                $language->setCurrentLanguage((int)$_REQUEST['langID']);
            }
        }
        if (!$parameters->getParameter("sef") || preg_match('/index\.php$/', $_SERVER['PHP_SELF'])) {
            if (basename($_SERVER["SCRIPT_FILENAME"], '.php') == "index") { // skip calls to php ajax helper scripts
                $symbol = ($_SERVER['QUERY_STRING']) ? "&" : NULL;
                $qvars = $variables->parseQuery($_SERVER['QUERY_STRING']);
                if (!array_key_exists("langID", $qvars)) {
                    $thisURL = "index.php?" . $_SERVER['QUERY_STRING'] . $symbol . "langID=" . $language->getCurrentLanguage();
                    if ($parameters->getParameter("sef")) {
                        header("location:" . $sef->url($thisURL, false));
                    } else {
                        header("location:" . $thisURL);
                    }
                    exit();
                }
            }
        }
    }
}

function getFPath($admin = false) {
    $parts = explode("/", $_SERVER["PHP_SELF"]);
    array_shift($parts); // the first element will be empty to we get rid of it
    array_pop($parts); // get rid of script name
    $fPath = implode("/", $parts);
    if ($admin) $fPath = str_replace("admin", NULL, $fPath);
    if (substr($fPath, -1) != "/") $fPath .= "/";
    $fPath = ($fPath == "/") ? NULL : $fPath;
    return $fPath;
}

function getBase() {
    global $domain_name;
    if ($_SERVER['SERVER_NAME'] == "localhost") return str_replace("localhost", NULL, $domain_name) . "/";
    global $domainPath;
    $www = preg_match('/^www/', $_SERVER['SERVER_NAME']) ? 'www.' : NULL;
    $parts = explode("/", $_SERVER["PHP_SELF"]);
    array_shift($parts); // the first element will be empty to we get rid of it
    array_pop($parts); // get rid of script name
    $fPath = implode("/", $parts);
    $newPath = $domainPath . $fPath;
    if (substr($newPath, -1) != "/") $newPath .= "/";
    return $newPath; // for sef urls, must be a base tag in html HEAD with this value
}

function t($key, $langID = NULL, $context = NULL) { // shortcut of $language->getText($key, $langID=NULL, $context=NULL)
    global $language;
    return $language->getText($key, $langID, $context);
}

function debug($var) {
    echo "<pre>";
    var_dump($var);
    echo "</pre>" . PHP_EOL;
}

function cleanUpConnectedUsers() {
    global $db;
    $now = time();
    $limit = 29;
    $new_limit = 30;
    $expired = $now - $limit;
    $q = "delete from connected_users where ts <= '$expired'";
    $db->query($q);
}

function array_htmlentities(&$elem, $mode = "encode") { // for encoding/decoding session values
    global $variables;
    if (!is_array($elem)) {
        if ($mode == "encode") {
            if ($variables->isMultiByte($elem)) {
                $elem = "__" . base64_encode($elem);
            }
            $elem = htmlspecialchars($elem);
        } else {
            if (preg_match('/^__/', $elem)) {
                $elem = base64_decode(substr($elem, 2));
            }
            $elem = html_entity_decode($elem);
        }
    } else {
        foreach ($elem as $key => $value) $elem[$key] = array_htmlentities($value, $mode);
    }
    return $elem;
}

function escapeJsonString($value) {
    $escapers = array("\\", "/", "\"", "\n", "\r", "\t", "\x08", "\x0c");
    $replacements = array("\\\\", "\\/", "\\\"", "\\n", "\\r", "\\t", "\\f", "\\b");
    $result = str_replace($escapers, $replacements, $value);
    return $result;
}

function loadGlobalFiles($type) {
    global $document;
    if (!$document) $document = new document();
    if ($type == "js") {
        $document->jsFiles = array(); // empty any existing js entries
        $jsGlobalFiles = array();
        $jsGlobalFilesTXT = $document->getTemplatePath() . "globalJsFiles.txt";
        if (is_file($jsGlobalFilesTXT)) {
            $entries = file($jsGlobalFilesTXT);
            foreach ($entries as $jsFile) $jsGlobalFiles[] = trim($jsFile);
        }
        $document->setJsFiles($jsGlobalFiles);
        $document->getJsFiles();
    }
    if ($type == "css") {
        $document->styles = array(); // empty any existing styles entries
        $cssGlobalFiles = array();
        $cssGlobalFilesTXT = $document->getTemplatePath() . "globalCSSFiles.txt";
        if (is_file($cssGlobalFilesTXT)) {
            $entries = file($cssGlobalFilesTXT);
            foreach ($entries as $cssFile) $cssGlobalFiles[] = trim($cssFile);
        }
        $document->setStyles($cssGlobalFiles);
        $document->getStyles();
    }
}

//email validation functions -----------------------------------------------------------------------------------------------------------------------
function win_checkdnsrr($host, $type = 'MX') { // checkdnsrr() support for Windows
    if (strtoupper(substr(PHP_OS, 0, 3)) != 'WIN') return;
    if (empty($host)) return;
    $types = array('A', 'MX', 'NS', 'SOA', 'PTR', 'CNAME', 'AAAA', 'A6', 'SRV', 'NAPTR', 'TXT', 'ANY');
    if (!in_array($type, $types)) {
        user_error("checkdnsrr() Type '$type' not supported", E_USER_WARNING);
        return;
    }
    @exec('nslookup -type=' . $type . ' ' . escapeshellcmd($host), $output);
    foreach ($output as $line) {
        if (preg_match('/^' . $host . '/', $line)) return true;
    }
}

if (!function_exists('checkdnsrr')) {
    function checkdnsrr($host, $type = 'MX') {
        return win_checkdnsrr($host, $type);
    }
}

function validEmail($email) {
    $isValid = true;
    $atIndex = strrpos($email, "@");
    if (is_bool($atIndex) && !$atIndex) {
        $isValid = false;
    } else {
        $domain = substr($email, $atIndex + 1);
        $local = substr($email, 0, $atIndex);
        $localLen = strlen($local);
        $domainLen = strlen($domain);
        if ($localLen < 1 || $localLen > 64) {
            $isValid = false;
        } else if ($domainLen < 1 || $domainLen > 255) {
            $isValid = false;
        } else if ($local[0] == '.' || $local[$localLen - 1] == '.') {
            $isValid = false;
        } else if (preg_match('/\\.\\./', $local)) {
            $isValid = false;
        } else if (!preg_match('/^[A-Za-z0-9\\-\\.]+$/', $domain)) {
            $isValid = false;
        } else if (preg_match('/\\.\\./', $domain)) {
            $isValid = false;
        } else if (!preg_match('/^(\\\\.|[A-Za-z0-9!#%&`_=\\/$\'*+?^{}|~.-])+$/', str_replace("\\\\", "", $local))) {
            if (!preg_match('/^"(\\\\"|[^"])+"$/', str_replace("\\\\", "", $local))) {
                $isValid = false;
            }
        }
        if ($isValid && !(checkdnsrr($domain))) {
            $isValid = false;
        }
    }
    return $isValid;
}

// end of email validation functions -----------------------------------------------------------------------------------------------------------------------

if (!function_exists("get_data")) {
    function get_data($url) {
        $ch = curl_init();
        $timeout = 1;
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_POST, 1);
        $data = curl_exec($ch);
        curl_close($ch);
        return $data;
    }
}

if (!function_exists('mb_ucfirst')) {
    function mb_ucfirst($str, $encoding = "UTF-8", $lower_str_end = false) {
        $first_letter = mb_strtoupper(mb_substr($str, 0, 1, $encoding), $encoding);
        $str_end = "";
        if ($lower_str_end) {
            $str_end = mb_strtolower(mb_substr($str, 1, mb_strlen($str, $encoding), $encoding), $encoding);
        } else {
            $str_end = mb_substr($str, 1, mb_strlen($str, $encoding), $encoding);
        }
        $str = $first_letter . $str_end;
        return $str;
    }
}

if (!function_exists('view')) {
    function view($viewMethod, $args) {
        require_once("library/view.php");
        $thisView = new view($viewMethod, $args);
        return $thisView->getViewData();
    }
}

function getLangCodes() {
    // reference: https://www.w3schools.com/tags/ref_language_codes.asp
    return array("gr" => "el", "en" => "en", "de" => "de", "it" => "it", "fr" => "fr", "es" => "es", "ru" => "ru", "ro" => "ro", "ar" => "ar", "tr" => "tr", "ch" => "zh", "zh" => "zh", "jp" => "ja", "ja" => "ja", 'ab' => 'ab', 'aa' => 'aa', 'af' => 'af', 'ak' => 'ak', 'sq' => 'sq', 'am' => 'am', 'an' => 'an', 'hy' => 'hy', 'as' => 'as', 'av' => 'av', 'ae' => 'ae', 'ay' => 'ay', 'az' => 'az', 'bm' => 'bm', 'ba' => 'ba', 'eu' => 'eu', 'be' => 'be', 'bn' => 'bn', 'bh' => 'bh', 'bi' => 'bi', 'bs' => 'bs', 'br' => 'br', 'bg' => 'bg', 'my' => 'my', 'ca' => 'ca', 'ch' => 'ch', 'ce' => 'ce', 'ny' => 'ny', 'cv' => 'cv', 'kw' => 'kw', 'co' => 'co', 'cr' => 'cr', 'hr' => 'hr', 'cs' => 'cs', 'da' => 'da', 'dv' => 'dv', 'nl' => 'nl', 'dz' => 'dz', 'eo' => 'eo', 'et' => 'et', 'ee' => 'ee', 'fo' => 'fo', 'fj' => 'fj', 'fi' => 'fi', 'ff' => 'ff', 'gl' => 'gl', 'gd' => 'gd', 'gv' => 'gv', 'ka' => 'ka', 'kl' => 'kl', 'gn' => 'gn', 'gu' => 'gu', 'ht' => 'ht', 'ha' => 'ha', 'he' => 'he', 'hz' => 'hz', 'hi' => 'hi', 'ho' => 'ho', 'hu' => 'hu', 'is' => 'is', 'io' => 'io', 'ig' => 'ig', 'id' => 'id', 'ia' => 'ia', 'ie' => 'ie', 'iu' => 'iu', 'ik' => 'ik', 'ga' => 'ga', 'kl' => 'kl', 'kn' => 'kn', 'kr' => 'kr', 'ks' => 'ks', 'kk' => 'kk', 'km' => 'km', 'ki' => 'ki', 'rw' => 'rw', 'rn' => 'rn', 'ky' => 'ky', 'kv' => 'kv', 'kg' => 'kg', 'ko' => 'ko', 'ku' => 'ku', 'kj' => 'kj', 'lo' => 'lo', 'la' => 'la', 'lv' => 'lv', 'li' => 'li', 'ln' => 'ln', 'lt' => 'lt', 'lu' => 'lu', 'lg' => 'lg', 'lb' => 'lb', 'gv' => 'gv', 'mk' => 'mk', 'mg' => 'mg', 'ms' => 'ms', 'ml' => 'ml', 'mt' => 'mt', 'mi' => 'mi', 'mr' => 'mr', 'mh' => 'mh', 'mo' => 'mo', 'mn' => 'mn', 'na' => 'na', 'nv' => 'nv', 'ng' => 'ng', 'nd' => 'nd', 'ne' => 'ne', 'no' => 'no', 'ii' => 'ii', 'oc' => 'oc', 'oj' => 'oj', 'cu' => 'cu', 'or' => 'or', 'om' => 'om', 'os' => 'os', 'pi' => 'pi', 'ps' => 'ps', 'fa' => 'fa', 'pl' => 'pl', 'pt' => 'pt', 'pa' => 'pa', 'qu' => 'qu', 'rm' => 'rm', 'se' => 'se', 'sm' => 'sm', 'sg' => 'sg', 'sa' => 'sa', 'sr' => 'sr', 'sh' => 'sh', 'st' => 'st', 'tn' => 'tn', 'sn' => 'sn', 'ii' => 'ii', 'sd' => 'sd', 'si' => 'si', 'ss' => 'ss', 'sk' => 'sk', 'sl' => 'sl', 'so' => 'so', 'nr' => 'nr', 'su' => 'su', 'sw' => 'sw', 'ss' => 'ss', 'sv' => 'sv', 'tl' => 'tl', 'ty' => 'ty', 'tg' => 'tg', 'ta' => 'ta', 'tt' => 'tt', 'te' => 'te', 'th' => 'th', 'bo' => 'bo', 'ti' => 'ti', 'to' => 'to', 'ts' => 'ts', 'tk' => 'tk', 'tw' => 'tw', 'ug' => 'ug', 'uk' => 'uk', 'ur' => 'ur', 'uz' => 'uz', 've' => 've', 'vi' => 'vi', 'vo' => 'vo', 'wa' => 'wa', 'cy' => 'cy', 'wo' => 'wo', 'fy' => 'fy', 'xh' => 'xh', 'yi' => 'yi', 'yo' => 'yo', 'za' => 'za', 'zu' => 'zu');
}

/*******************decode unicode text - Usage: $decoded = unicode_decode('\u003c');*******************/
function replace_unicode_escape_sequence($match) {
    return mb_convert_encoding(pack('H*', $match[1]), 'UTF-8', 'UCS-2BE');
}

function unicode_decode($str) {
    return preg_replace_callback('/\\\\u([0-9a-f]{4})/i', 'replace_unicode_escape_sequence', $str);
}
/*************************************************************************************/


?>